/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile;

import com.tomwheeler.example.tdlfile.intf.ListContentListener;
import com.tomwheeler.example.tdlfile.intf.TodoList;
import com.tomwheeler.example.tdlfile.intf.TodoListItem;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.swing.Action;
import javax.swing.event.ChangeEvent;
import org.openide.actions.EditAction;
import org.openide.actions.PropertiesAction;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObject;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.nodes.NodeAdapter;
import org.openide.nodes.NodeEvent;
import org.openide.util.Lookup;
import org.openide.util.actions.SystemAction;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.Lookups;

/**
 * <p>
 * This defines the NetBeans node that represents a TodoList.  It is a 
 * a DataNode, which is a type of Node that represents a DataObject, which
 * itself represents one (and in some cases, more than one) FileObject. The
 * bulk of it was generated by the File Type wizard in the NetBeans 5.0 IDE, 
 * but I had to add a lot of code specific to my needs.
 * </p>
 * 
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public class TodoListFileDataNode extends DataNode {
    
    private static final String IMAGE_ICON_BASE = "com/tomwheeler/example/tdlfile/resources/tdlfile.png";
    
    public TodoListFileDataNode(TodoListFileDataObject obj, InstanceContent ic) {
        super(obj, new TodoListFileChildren(obj), new AbstractLookup(ic));
        setIconBaseWithExtension(IMAGE_ICON_BASE);
        
        // since we overrode the normal behavior of getCookie(Class) in our DataObject 
        // (i.e., so that we can use lookups, we are not using a CookieSet), we must add
        // this code to prevent a possible AssertionFailedError described here:
        // see http://www.netbeans.org/issues/show_bug.cgi?id=68285
        obj.addPropertyChangeListener(new PropertyChangeListener() {   
            public void propertyChange(PropertyChangeEvent e) {      
                if (DataObject.PROP_MODIFIED.equals(e.getPropertyName())) {         
                    fireCookieChange();      
                }   
            }
        }); 
    }
    
    public Action[] getActions(boolean context) {
        // these represent the items that show up in the context menu for
        // the node.  You usually want to call the superclass implementation
        // and add those to the array first before adding your own actions,
        // unless your intention is to override (not provide) those actions.
        
        List allActions = null;
        // a null item here is rendered as an item separator in the menu
        Action[] parentActions = super.getActions(context);
        Action[] myActions = new Action[]{
            null,
            SystemAction.get(AddItemAction.class),
            SystemAction.get(PropertiesAction.class),
            SystemAction.get(EditAction.class),
        };

        allActions = new ArrayList();
        if (parentActions.length > 0) {
            allActions.addAll(Arrays.asList(parentActions));
        }
        allActions.addAll(Arrays.asList(myActions));
        return (Action[]) allActions.toArray(new Action[0]);
    }
    
    /** specifies what happens on double-click */
    public Action getPreferredAction() {
        return SystemAction.get(EditAction.class);
    }
    
    /** this class is responsible for constructing the child nodes (which are instances of SingleItemNode) */
    private static class TodoListFileChildren extends Children.Keys {
        private TodoListFileDataObject dataObj;
        
        TodoListFileChildren(TodoListFileDataObject dataObj) {
            this.dataObj = dataObj;
            
            // recalculate the child nodes whenever the content of the TodoList changes
            TodoList tdl = getTodoList();
            tdl.addListContentListener(new ListContentListener() {
                public void itemAdded(ChangeEvent e) {
                    refreshList();
                }
                public void itemRemoved(ChangeEvent e) {
                    refreshList();
                }
            });
        }
        
        /** local convenience method to get the TodoList from the data object */
        private TodoList getTodoList() {
            // Normally, the TodoList for a node should accessed via the node's lookup.
            // This allows us to get to it without the DataObject being visible to other 
            // modules and without us having to know any specific methods to access it.  
            // Since a DataNode will always be tightly coupled with the DataObject it 
            // represents, accessing it this way is overkill.  Plus, it would not work
            // anyway since this is called from TodoListFileChildren constructor, which
            // is in turn called from the TodoListFileDataNodeConstructor, which would
            // cause a StackOverflowError.
            //
            // Also be sure you call getNodeDelegate() here and NOT createNodeDelegate(), 
            // since you just want to access the node -- not create a new one.
            return dataObj.getTodoList();
        }
        
        /**
         * This method must determines which child nodes should be shown and
         * then calls setKeys(), passing in a collection of whatever objects
         * are used to construct the child nodes.  In our case, we have a child
         * for every item in the TodoList, and the keys are the TodoListItem
         * instances themselves.
         */
        protected void addNotify() {
            refreshList();
        }
        
        /**
         * This is called whenever the nodes are no longer needed.  You might
         * do some cleanup to save memory, but calling setKeys() with an
         * empty collection is enough for our needs.
         */
        protected void removeNotify() {
            setKeys(Collections.EMPTY_SET);
        }
        
        protected Node[] createNodes(Object key) {
            TodoListItem item = (TodoListItem) key;
            
            // we must ensure that the SingleItemNode contains the TodoListItem that 
            // it represents so that other classes will be able to access it from 
            // its lookup.
            Lookup myLookup = Lookups.fixed(new Object[] { item });
            SingleItemNode todoItemNode = new SingleItemNode(item, myLookup);

            // make sure that changes to the nodes get reflected as modifications to the TodoList
            todoItemNode.addNodeListener(new NodeAdapter() {
                // node was deleted; find out which TodoListItem it corresponds to 
                // and remove it from the TodoList
                public void nodeDestroyed(NodeEvent ev) {
                    Lookup lookup = ev.getNode().getLookup();
                    TodoListItem item = (TodoListItem)lookup.lookup(TodoListItem.class);
                    assert(item != null);
                    
                    TodoList parent = item.getTodoList();
                    parent.removeItem(item);
                    dataObj.setModified(true);
                }
                
                // node was renamed; find the item corresponding to it and change its name too
                public void propertyChange(PropertyChangeEvent evt) {
                    if (Node.PROP_NAME.equals(evt.getPropertyName())) {
                        Node node = (Node)evt.getSource();
                        Lookup lookup = node.getLookup();
                        TodoListItem item = (TodoListItem)lookup.lookup(TodoListItem.class);
                        assert(item != null);
                        item.setName(node.getName());
                        dataObj.setModified(true);
                    }
                }
            });

            item.addPropertyChangeListener(new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    // mark the file as needing to be saved whenever a TodoList
                    // item is modified
                    dataObj.setModified(true);                    
                }
            });
            
            return new Node[] { todoItemNode };
        }
        
        private void refreshList() {
            // get all the items currently in the list
            // be sure you call getNodeDelegate() here and NOT createNodeDelegate()
            TodoListFileDataNode node = (TodoListFileDataNode)dataObj.getNodeDelegate();
            TodoList tdl = (TodoList)node.getLookup().lookup(TodoList.class);
            if (tdl == null) {
                System.err.println("ERROR: Todo list in lookup was null");
                return;
            }
            
            Set<TodoListItem> items = new HashSet<TodoListItem>();
            for (Iterator it = tdl.getItems(); it.hasNext();) {
                TodoListItem item = (TodoListItem) it.next();
                items.add(item);
            }
            
            setKeys(items);
        }
    }
}
